
from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _


class CategoriesAppConfig(AppConfig):

    name = 'categories'
    verbose_name = _('Categories')


default_app_config = 'categories.CategoriesAppConfig'




from categories import defaults


class CategorySettings(object):

    IS_CATEGORY_LOGO_ENABLED = defaults.IS_CATEGORY_LOGO_ENABLED
    IS_CATEGORY_CODE_ENABLED = defaults.IS_CATEGORY_CODE_ENABLED
    IS_CATEGORY_ICON_ENABLED = defaults.IS_CATEGORY_ICON_ENABLED
    IS_CATEGORY_DESCRIPTION_ENABLED = defaults.IS_CATEGORY_DESCRIPTION_ENABLED

    @property
    def INSTALLED_APPS(self):
        return super().INSTALLED_APPS + ['categories']




from modeltranslation.translator import translator

from categories.models import Category
from categories import config


def get_fields():

    fields = ['name']

    if config.IS_CATEGORY_DESCRIPTION_ENABLED:
        fields += ['description']

    return fields


translator.register(Category, fields=get_fields())




from django import template
from django.apps import apps


register = template.Library()


@register.simple_tag()
def get_categories():
    return apps.get_model('categories', 'Category').objects.all()




from django.apps import apps
from django.db import models
from django.contrib import admin
from django.utils.translation import ugettext_lazy as _

from cap.decorators import short_description, template_list_item

from categories.models import Category
from categories import config


def get_admin_base_class():

    if apps.is_installed('modeltranslation'):
        from modeltranslation.admin import TranslationAdmin
        return TranslationAdmin

    return admin.ModelAdmin


def get_list_display_items():

    result = ['name']

    if hasattr(Category, 'products'):
        result += ['product_count']

    if config.IS_CATEGORY_LOGO_ENABLED:
        result += ['get_preview']

    return result


def get_formfield_overrides():

    if apps.is_installed('ckeditor'):
        from ckeditor.widgets import CKEditorWidget
        return {
            models.TextField: {'widget': CKEditorWidget}
        }

    return {}


@admin.register(Category)
class CategoryAdmin(get_admin_base_class()):

    list_display = get_list_display_items()

    formfield_overrides = get_formfield_overrides()

    @template_list_item('admin/list_item_preview.html', _('Preview'))
    def get_preview(self, item):
        return {'file': item.logo}

    @short_description(_('Product count'))
    def product_count(self, item):
        return item.products.count()




from django.conf import settings

from categories import defaults


IS_CATEGORY_LOGO_ENABLED = getattr(
    settings,
    'IS_CATEGORY_LOGO_ENABLED',
    defaults.IS_CATEGORY_LOGO_ENABLED)

IS_CATEGORY_CODE_ENABLED = getattr(
    settings,
    'IS_CATEGORY_CODE_ENABLED',
    defaults.IS_CATEGORY_CODE_ENABLED)

IS_CATEGORY_ICON_ENABLED = getattr(
    settings,
    'IS_CATEGORY_ICON_ENABLED',
    defaults.IS_CATEGORY_ICON_ENABLED)

IS_CATEGORY_DESCRIPTION_ENABLED = getattr(
    settings,
    'IS_CATEGORY_DESCRIPTION_ENABLED',
    defaults.IS_CATEGORY_DESCRIPTION_ENABLED)




from django.db import models
from django.urls import reverse_lazy
from django.utils.translation import ugettext_lazy as _

from slugify import slugify_url

from categories import config


class Category(models.Model):

    name = models.CharField(
        _('Category name'),
        max_length=255)

    if config.IS_CATEGORY_LOGO_ENABLED:
        logo = models.ImageField(
            _('Logo'),
            upload_to='categories',
            blank=True,
            null=True,
            max_length=255)

    if config.IS_CATEGORY_CODE_ENABLED:
        code = models.CharField(
            _('Code'),
            max_length=255,
            blank=True)

    if config.IS_CATEGORY_ICON_ENABLED:
        icon = models.CharField(
            _('Icon'),
            max_length=255,
            blank=True)

    if config.IS_CATEGORY_DESCRIPTION_ENABLED:
        description = models.TextField(
            _('Description'),
            blank=True,
            max_length=4096)

    def __str__(self):
        return self.name

    def get_absolute_url(self):
        return reverse_lazy('products:list', args=[self.slug, self.id])

    @property
    def slug(self):
        return slugify_url(self.name or 'category', separator='_')

    class Meta:
        verbose_name = _('Category')
        verbose_name_plural = _('Categories')


class CategoryField(models.ForeignKey):

    def __init__(
            self,
            to='categories.Category',
            verbose_name=_('Category'),
            on_delete=models.CASCADE,
            *args, **kwargs):

        super(CategoryField, self).__init__(
            to,
            verbose_name=verbose_name,
            on_delete=on_delete,
            *args, **kwargs)



# Generated by Django 2.0.9 on 2020-03-24 20:28

from django.db import migrations, models

from categories import config


def get_fields():

    fields = [
        ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
        ('name', models.CharField(max_length=255, verbose_name='Category name')),
    ]

    if config.IS_CATEGORY_LOGO_ENABLED:
        fields += [('logo', models.ImageField(blank=True, max_length=255, null=True, upload_to='categories', verbose_name='Logo'))]

    if config.IS_CATEGORY_CODE_ENABLED:
        fields += [('code', models.CharField(blank=True, max_length=255, verbose_name='Code'))]

    if config.IS_CATEGORY_ICON_ENABLED:
        fields += [('icon', models.CharField(blank=True, max_length=255, verbose_name='Icon'))]

    if config.IS_CATEGORY_DESCRIPTION_ENABLED:
        fields += [('description', models.TextField(blank=True, max_length=4096, verbose_name='Description'))]

    return fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Category',
            fields=get_fields(),
            options={
                'verbose_name': 'Category',
                'verbose_name_plural': 'Categories',
            },
        ),
    ]
